<%
'===============================================
' CactuShop ASP Shopping Cart
' 1999-2007 Cactusoft International FZ-LLC
' www.cactusoft.com
'===============================================
' All rights reserved.							
' Use of this code is covered by the terms and	
' conditions in the license agreement. No		
' unauthorized duplication or distribution is	
' permitted. Cactusoft's copyright notices must	
' remain in the ASP sections of the code.		
'===============================================

'Size of the random string for the session code
Const CODE_LENGTH = 6

'Name of the variable used to pass in the querystring/form
Const VARIABLE_NAME = "s"

'Potential values for 'usingcookies'
Const USING_COOKIES_YES = "yes"
Const USING_COOKIES_NO = "no"
Const USING_COOKIES_BACKUP = "backup"

'Name of the table in the database
SESSION_TABLENAME = "tblCactuShop" & TABLE_PREFIX & "Sessions"
SESSIONVALUES_TABLENAME = "tblCactuShop" & TABLE_PREFIX & "SessionValues"

Dim COOKIE_NAME

Dim DEFAULT_SESSION_EXPIRY

'-----------------------------------------------
'CACTUSESSION VALUE
'The session value class represents one name/value
'pair in the database. It also includes an expiry
'date (and created/modified dates, id, and link to
'the session id).
'Update to the DB (insert, modify and delete) are
'done on class termination. 
'Record is only edited if a change has taken place
'-----------------------------------------------
Class CactuSessionValue

	'Original settings - useful to know if we should update it
	'when the class terminates
	Private strOriginalValue	
	Private numOriginalExpiry
	
	'ID and name are not accessible, but required internally
	Private SESSV_ID
	Private SESSV_SessionID 
	Private SESSV_Name	
		
	'This can be altered outside the class
	Public Value				'The value held
	Public Expiry				'Minutes until the value expires
	Public Deleted				'Whether to flag the name/value for deletion
	
	Private Sub Class_Initialize()
		'Set the original values
	End Sub
	
	'These procedures set up the values
	Public Sub SetUp(numID, numSessionID, strName, strValue, numExpiry)
		SESSV_ID = numID
		SESSV_SessionID = numSessionID
		SESSV_Name = strName
		Value = strValue
		Expiry = NumSafe(numExpiry)
		
		'Set up the defaults
		strOriginalValue = Value
		numOriginalExpiry = Expiry
	End Sub
	
	Public Sub SetUpAsNew(numSessionID, strName, strValue, numExpiry)
		Call SetUp(0, numSessionID, strName, strValue, numExpiry)
	End Sub
	
	Private Sub Class_Terminate()
		'If it's been flagged for deletion, or it exists but 
		'it's value is blank, delete it from the database
		Value = Left(Value, 255)
		If Deleted or (Value = "" and SESSV_ID <> 0) then
			'Theoretically it may not existing in the DB yet, so don't do
			'anything.
			If SESSV_ID > 0 then
				strQuery = strDeleteStatementSyntax & " " & SESSIONVALUES_TABLENAME & " WHERE SESSV_ID = " & SESSV_ID
				Call ExecuteNonQuery(strQuery)
			End If
			
		'Add if this is a new record - otherwise update it
		'only if the data has changed from the original
		ElseIf SESSV_ID = 0 then
			'If it's blank, don't insert
			If Not Value = "" then
				strQuery = "INSERT INTO " & SESSIONVALUES_TABLENAME & " (SESSV_SessionID, SESSV_Name, SESSV_Value, SESSV_Expiry) VALUES (" &_
					SESSV_SessionID &_
					"," & strUnicode & "'" & sqlsafe(SESSV_Name) & "'" &_
					"," & strUnicode & "'" & sqlsafe(Value) & "'" &_
					"," & Expiry &_
					")"
				Call ExecuteNonQuery(strQuery)
							
			End If
			
		'If the values has changed from the original value
		ElseIf Value <> strOriginalValue or Expiry <> numOriginalExpiry then
			strQuery = "UPDATE " & SESSIONVALUES_TABLENAME & " SET " &_
				"SESSV_Value=" & strUnicode & "'" & SqlSafe(Value) & "'" &_
				",SESSV_Expiry='" & SqlSafe(Expiry) & "'" &_
				" WHERE SESSV_ID = " & SESSV_ID
			Call ExecuteNonQuery(strQuery)	
		End If
		
	End Sub
		
End Class


'-----------------------------------------------
'CACTUSESSION
'This represents one session. It deals with all
'the creation and updating of the session, and
'contains a dictionary of all the name/value
'pairs in the session.
'Initialise and Terminate functions deal with
'the database, so records are only updated 
'when the object is killed.
'
'You need to know the following functions:
'Add		- adds or edits a name/value pair
'Edit		- Same as add pretty much
'Delete		- Tags an item for deletion
'SetExpiry	- Sets the expiry for a value
'
'Exists		- Gets whether item exists
'Value		- Gets the value of an item
'Expiry		- Gets the expiry of an item

'These functions are used to pass on the session 
'ID between pages. Must be used on all links, 
'redirects, forms, meta redirects
'QS		- Gets the string to add to a querystring 
'Form		- Same, for a form
'-----------------------------------------------
Class CactuSession

	'Dictionary object to contain all values
	Private objItems
	
	'These relate directly to the database values. Keep 'em
	'private as we don't want others pages messing with it.
	Private SESS_ID
	Private SESS_Code
	Private strUsingCookies
	Private blnIsBrowser
	
	Public Property Get IsBrowser
		IsBrowser = blnIsBrowser
	End Property
	
	'-----------------------------------------------------
	'Returns the session querystring & form to be passed 
	'between pages. Return blank if we're using cookies
	'and not during checkout (where we could be on https)
	'-----------------------------------------------------
	Public Property Get Code()
		Code = SESS_Code & SESS_ID
	End Property
	
	Public Property Get QS()
		If blnIsBrowser then
			If strUsingCookies <> USING_COOKIES_YES or blnMenuDisabled then	
				QS = VARIABLE_NAME & "=" & Code & "&amp;"
			End If
		End If
	End Property
	
	Public Property Get Form()
		If blnIsBrowser and (strUsingCookies <> USING_COOKIES_YES or blnMenuDisabled) then
			Form = "<input type=""hidden"" name=""" & VARIABLE_NAME & """ value=""" & Code & """ />"
		End If
	End Property
	
	'IP can be useful in various pages so we'll make it public
	Public IP
		
	'-----------------------------------------------------
	'FUNCTIONS TO CONTROL THE SESSION NAME/VALUES
	'Public functions to control the items. All very 
	'defensive - i.e. adding an item that exists edits it,
	'editing new items get added
	'-----------------------------------------------------
	'Adds a new name/value pair
	Public Sub Add(strName, strValue)
		If blnIsBrowser then
			If objItems.Exists(Lcase(strName)) then
				Call Edit(strName, strValue)
			Else
				Call AddItem(strName, strValue)
			End If
		End If
	End Sub
	
	'Edits a new name/value pair
	Public Sub Edit(strName, strValue)
		If blnIsBrowser then
			If Not objItems.Exists(Lcase(strName)) then
				Call AddItem(strName, strValue)
			Else
				'Change the value, and make sure it's
				'not tagged for deletion
				objItems.Item(Lcase(strName)).Value = strValue
				objItems.Item(LCase(strName)).Deleted = false
			End If
		End If
	End Sub
	
	'Alters the expiry for this item
	Public Sub SetExpiry(strName, numExpiry)
		If blnIsBrowser then
			If not objItems.Exists(LCase(strName)) then Call AddItem(strName, "")
			objItems.Item(LCase(strName)).Expiry = numExpiry
		End If
	End Sub
	
	'Deletes an item
	Public Sub Delete(strName)
		If blnIsBrowser then
			If objItems.Exists(LCase(strName)) then 
				objItems.Item(LCase(strName)).Deleted = True
			End If
		End If
	End Sub	
	
	'Gets whether an item exists. Rarely needed as add/edit is defensive
	'Must not only be in the dictionary object, but also not tagged
	'for deletion and not blank in value
	Public Function Exists(StrName)
		if blnIsBrowser then
			Exists = objItems.Exists(LCase(strName))
			If Exists = true then Exists = (not objItems(LCase(strName)).Deleted and not objItems(LCase(strName)).Value = "")
		End if
	End Function
	
	'Gets the value of an item. Creates it if it doesn't exist.
	'Return blank if it's been flagged for deletion
	Public Function Value(strName)
		If blnIsBrowser then
			If not objItems.Exists(LCase(strName)) then Call AddItem(strName, "")
			Value = IIf(objItems(LCase(strName)).Deleted, "", objItems(LCase(strName)).Value)
		End if
	End Function

	'Gets the expiry for an item
	Public Function Expiry(strName)
		If blnIsBrowser then
			If not objItems.Exists(LCase(strName)) then Call AddItem(strName, "")
			Expiry = objItems(LCase(strName)).Expiry
		End If
	End Function
	'-----------------------------------------------------

			
	'Adds a new value. This is private as it's not defensive.
	Private Function AddItem(strName, strValue)
		Call objItems.Add(LCase(strName), New CactuSessionValue)
		Call objItems.Item(LCase(strName)).SetUpAsNew(SESS_ID, LCase(strName), strValue, DEFAULT_SESSION_EXPIRY)
	End Function
	
	'Initialise function. Pulls out, or creates a new, session. Sets up the items
	'dictionary and adds in any existing values from the database.
	private sub Class_Initialize()	
		blnIsBrowser = false
		strHTTPUserAgent = LCase(request.ServerVariables("HTTP_USER_AGENT"))
		If Instr(strHTTPUserAgent, "msie") > 0 or Instr(strHTTPUserAgent, "gecko") > 0 or Instr(strHTTPUserAgent, "opera") > 0 or Instr(strHTTPUserAgent, "netscape") > 0 or Instr(strHTTPUserAgent, "safari") > 0 then
			blnIsBrowser = true
				
			'Set related config settings
			COOKIE_NAME = GetAppVar("cookiename")
			DEFAULT_SESSION_EXPIRY = GetAppVar("sessionexpiry")
			strUsingCookies = GetAppVar("usecookies")
			
			'Create dictionary
			Set objItems = Server.CreateObject("Scripting.Dictionary")
						
			'Try and find a session id somewhere. The cookie takes preference (so
			'if a user goes to a page in the history with an old ID, they won't
			'switch sessions and see an old basket, someone elses login etc)
			strValue = ""
			If strUsingCookies <> USING_COOKIES_NO then
				strValue = request.Cookies(COOKIE_NAME)(VARIABLE_NAME) & ""
			End If
			If strValue = "" then 
				If blnUploading then
					strValue = Uploader.Form(VARIABLE_NAME)
				Else
					strValue = request.QueryString(VARIABLE_NAME)
					If strValue = "" then strValue = request.Form(VARIABLE_NAME)
				End If
			End If
			
			'Still not got it? Try from fakeurl if that's there
			If strValue = "" then
				'If we're not using cookies, first folder is the sessionQS
				If GetAppVar("usecookies") = "no" or GetAppVar("usecookies") = "backup" then
					strQueryString = request.QueryString
					If Left(strQueryString, 4) = "404;" then
						'Get the first folder after the webshopurl - because the URL may contain the port # (http://www.mysite.com:80/folder/...)
						'then we count how many slashes in the webshopurl, count that off and get the folder after that point
						strAppWebShopURL = GetAppVar("webshopurl")
						numTotalSlashesInURL = Occurrences(strAppWebShopURL, "/")
						numSlashPos = 0
						For i = 1 to numTotalSlashesInURL
							numSlashPos = Instr(numSlashPos + 1, strQueryString, "/")
						Next
						strTrimmedQS = Mid(strQueryString, numSlashPos + 1)
						If InStr(strTrimmedQS, "/") > 0 then
							strValue = Left(strTrimmedQS, InStr(strTrimmedQS, "/") - 1)
						End If
					End If
				End If
			End If
			
			'Check it's the right length
			If len(strValue) < CODE_LENGTH + 1 then
				SESS_ID = 0
				SESS_Code = ""
			Else
				'Get the random code prefix, followed by the ID
				SESS_Code = Left(strValue, CODE_LENGTH)
				SESS_ID = NumSafe(Mid(strValue, CODE_LENGTH + 1))
			End If
			
			IP = request.ServerVariables("REMOTE_ADDR")
			
			'We set up the session. If we've got a session code then try and pull it
			'out of the database. Keep open the recordset for laters
			If SESS_ID > 0 then
				Select Case strDatabaseType
					Case "access"
						strDateAdd = "DATEADD('n', SESS_Expiry, SESS_DateLastUpdated) >= "
					Case "mysql"
						strDateAdd = "DATE_ADD(SESS_DateLastUpdated, interval SESS_EXPIRY minute) >= " 	
					Case "mssql"
						strDateAdd = "DATEADD(n, SESS_Expiry, SESS_DateLastUpdated) >= "
				End Select
				strDateAdd = strDateAdd & strDateDelimiter & ReverseDate(NowOffset) & strDateDelimiter
			
				strQuery = "SELECT SESS_ID, SESS_Code, SESS_DateLastUpdated, sv.* FROM " & SESSION_TABLENAME & " s LEFT JOIN " & SESSIONVALUES_TABLENAME & " sv ON s.SESS_ID = sv.SESSV_SessionID WHERE" &_
					" SESS_ID = " & SESS_ID &_
					" AND SESS_Code = " & strUnicode & "'" & sqlsafe(SESS_Code) & "'" &_
					" AND " & strDateAdd

					'removed this line since AOL proxy serves different IPs and it seems to cause problems then
					'" AND SESS_IP = '" & sqlsafe(IP) & "'" &_

							
				Call ExecuteRS(strQuery, objRecordSet)
				If Not (objRecordSet.BOF and objRecordSet.EOF) then
					SESS_Code = objRecordSet("SESS_Code")
					SESS_DateLastUpdated = objRecordSet("SESS_DateLastUpdated")
				Else
					SESS_Code = ""
					SESS_ID = 0
					objRecordSet.Close
				end if
			end if
			
			'If we haven't got a session, need to create a new one. Need to do an
			'insert now so that we get the database ID
			If SESS_ID = 0 then
				SESS_Code = GetRandomString(CODE_LENGTH)
				SESS_DateCreated = NowOffset()
				SESS_DateLastUpdated = SESS_DateCreated
				SESS_Values = ""
				SESS_Expiry = DEFAULT_SESSION_EXPIRY
				strQuery = "INSERT INTO " & SESSION_TABLENAME & " (SESS_Code, SESS_IP, SESS_DateCreated, SESS_DateLastUpdated, SESS_Expiry) VALUES (" &_
					strUnicode & "'" & sqlsafe(SESS_Code) & "'" &_
					",'" & sqlsafe(IP) & "'" &_
					"," & strDateDelimiter & ReverseDate(SESS_DateCreated) & strDateDelimiter &_
					"," & strDateDelimiter & ReverseDate(SESS_DateLastUpdated) & strDateDelimiter &_
					"," & SESS_Expiry &_
					")"
				Call ExecuteNonQuery(strQuery)

				'Get out the new ID
				strQuery = "SELECT SESS_ID FROM " & SESSION_TABLENAME & " WHERE SESS_Code = " & strUnicode & "'" & SESS_Code & "' AND SESS_DateCreated = " & strDateDelimiter & ReverseDate(SESS_DateCreated) & strDateDelimiter & " ORDER BY SESS_ID DESC"
				Call ExecuteRS(strQuery, objRecordSet)
				Do while objRecordSet.BOF and objRecordSet.EOF
					objRecordSet.Close
					Call ExecuteRS(strQuery, objRecordSet)
				Loop
				SESS_ID = objRecordSet("SESS_ID")
				objRecordSet.Close
			else

				'Loop through the recordset and set up all the items
				If Not isnull(objRecordSet("SESSV_ID")) Then
					Do While not objRecordSet.EOF
						'Get out the values
						SESSV_ID = objRecordSet("SESSV_ID")
						SESSV_SessionID = objRecordSet("SESSV_SessionID")
						SESSV_Name = objRecordSet("SESSV_Name")
						SESSV_Value = objRecordSet("SESSV_Value")
						SESSV_Expiry = objRecordSet("SESSV_Expiry")
						'If it's expired, delete
						If cdate(DateAdd("N", SESSV_Expiry, formatdatetime(SESS_DateLastUpdated,0))) < cdate(formatdatetime(nowoffset,0)) Then
						response.end
							strQuery = strDeleteStatementSyntax & " " & SESSIONVALUES_TABLENAME & " WHERE SESSV_ID = " & SESSV_ID
							Call ExecuteNonQuery(strQuery)
						
						'If this name already exists in the DB then delete
						ElseIf objItems.Exists(LCase(SESSV_Name)) then
							'Delete the extra record
							strQuery = strDeleteStatementSyntax & " " & SESSIONVALUES_TABLENAME & " WHERE SESSV_ID = " & SESSV_ID
							Call ExecuteNonQuery(strQuery)
						
						Else
							'Set up the value and add to our dictionary
							Call objItems.Add(LCase(SESSV_Name), New CactuSessionValue)
							Call objItems.Item(LCase(SESSV_Name)).SetUp(SESSV_ID, SESSV_SessionID, SESSV_Name, SESSV_Value, SESSV_Expiry)
						End If
						
						'Go to the next value
						objRecordSet.MoveNext
					Loop
				End If
				objRecordSet.Close
			End If
			
			'Set the starting values. We use this to tell whether we need to 
			'update the database later on
			strInitialValues = SESS_Values
			
			'Store the session code in a session-length cookie.
			If strUsingCookies <> USING_COOKIES_NO then
				response.Cookies(COOKIE_NAME)(VARIABLE_NAME) = SESS_Code & SESS_ID
			End If
		End If
	end sub 	
	 	
	 	
	'Termination class. Cleans everything up
	private sub Class_Terminate()	
		If blnIsBrowser then
			'Update all session values with the current date
			strQuery = "UPDATE " & SESSION_TABLENAME & " SET SESS_DateLastUpdated=" & strDateDelimiter & ReverseDate(NowOffset) & strDateDelimiter & " WHERE SESS_ID = " & SESS_ID
			Call ExecuteNonQuery(strQuery)
			
			'Close the dictionary object. This terminates all our
			'session value items		
			Set objItems = Nothing
		End If
	end sub
	
End class

%>
